/* -*- Javascript -*- */

/* global variables */

var config = null;
var file = null;
var export_filename = 'export_config.xml';

/* load function */

function load_config () {

  /* update status */
  update_status ('loading configuration table', 0);

  /* create request message */
  var params = 'action=load';
  var rxfunc = function (message) {
    var stat = message.getElementsByTagName ('stat')[0].firstChild.nodeValue;
    if (stat == 0) {
      var data = message.getElementsByTagName ('data')[0].firstChild.nodeValue;
      config = base64_decode (data);
    } else {

      /* update status on error */
      update_status ('error while loading configuration file (' + stat + ')', 2);
    }
  };

  ajax_post (params, rxfunc, 'cfg/func.php');

  return '';
}

/* create row in table */

function create_row (table, label, variable, value, type, default_value) {
  if (typeof type === 'undefined') type = 'string';
  if (typeof default_value === 'undefined') default_value = '';

  /* create row */
  var row_config = table.insertRow (table.rows.length);

  if (variable == '') {

    /* title */
    var cell_title_config = row_config.insertCell (0);
    cell_title_config.colSpan = 2;
    var title_config = document.createElement ('h3');
    title_config.innerHTML = label;
    cell_title_config.appendChild (title_config);

  } else {

    /* create label column */
    var cell_label_config = row_config.insertCell (0);
    cell_label_config.className = 'fit';
    var label_config = document.createElement ('label');
    label_config.innerHTML = label + ': ';
    label_config.forName = variable;
    cell_label_config.appendChild (label_config);

    /* create input column */
    var cell_input_config = row_config.insertCell (1);
    if (type == 'string')
      cell_input_config.className = 'expand';
    var input_config = create_input (variable, value, type, default_value);
    cell_input_config.appendChild (input_config);

  }

  return table;
}

/* create full table */

function create_table (xmltree, config) {

  var table = document.createElement ('table');
  table.className = 'table';
  table.id = 'config';

  var tab = config.split ('\n');
  for (var i in tab) {

    var row = tab[i].replace (/#.*/, '').split (';');
    if (row.length <= 1)
      continue;
    var element_list = row[0].split (',');
    var param = row[1];
    var description = row[2];
    var type = row[3];
    var default_value = row[4];

    if (element_list.length == 1) {

      create_row (table, description, '', '', '', '');

    } else {

      var element = find_element_from_xmltree (xmltree, element_list);
      if (!element)
        continue;

      var value = (param.length) ? element.getAttribute (param) : element.textContent;
      if (typeof value === 'undefined')
        continue;

      var name = get_tree_name (xmltree, element_list) + param;
      create_row (table, description, name, value, type, default_value);

    }

  }

  return table;
}

/* update xmltree */

function update_xmltree (xmltree, config) {

  var tab = config.split ('\n');
  for (var i in tab) {
 
    var row = tab[i].replace (/#.*/, '').split (';');
    if (row.length <= 1)
      continue;
    var element_list = row[0].split (',');
    var param = row[1];
    var type = row[3];

    var element = find_element_from_xmltree (xmltree, element_list);
    if (!element)
      continue;
 
    var name = get_tree_name (xmltree, element_list) + param;
    var ref = document.getElementById (name);
    if (ref) {
      var value = get_input_value (ref);
      if (param.length)
        element.setAttribute (param, value);
      else
        element.textContent = value;
    }
  }
}

/* import procedure */

function import_proc (data) {

  /* remove previous table */
  var table = document.getElementById ('config');
  if (!table) {
    update_status ('can\'t find previous table');
    return;
  }
  var father = table.parentNode;
  father.removeChild (table);

  /* save data file*/
  file = data;

  /* process xml data */
  var xmldata = new DOMParser ().parseFromString (file, 'text/xml');

  /* create new table */
  table = create_table (xmldata, config);

  /* add new table into html page */
  father.appendChild (table);

  /* table is updated */
  update_status ('import done', 0);
}

/* export raw database */

function export_proc () {

  /* look for table */
  var table = document.getElementById ('config');
  if (!table) {
    update_status ('can\'t find previous table');
    return;
  }

  /* process xml data */
  var xmldata = new DOMParser ().parseFromString (file, 'text/xml');

  /*  process element/argument tuplet */
  update_xmltree (xmldata, config);

  /* serialize data */
  var data = (new XMLSerializer ()).serializeToString (xmldata.documentElement);

  /* add static header */
  var header = '<?xml version="1.0" encoding="ISO-8859-1"?>\n' +
    '<!--\n     XML configuration file\n-->\n';

  return header + data;
}
